#!/usr/bin/env python

import gtk
import gtk.glade
import os
import pwd
import crypt
import random
import re
import commands
import locale
import string
import gettext
import signal

TRUE  = 1
FALSE = 0

domain = 'smbdcsetup'
gettext.bindtextdomain(domain, '/usr/share/locale')
gettext.textdomain(domain)
gtk.glade.bindtextdomain(domain, '/usr/share/locale')
_ = gettext.gettext

fname = '/usr/share/smbdcsetup/smbdcsetup.glade'
backdir = '/etc/smbdcsetup/backups/'
tempdir = '/etc/smbdcsetup/templates/'


def disp_err_dialog(self, str):
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
			       type=gtk.MESSAGE_ERROR,
			       buttons=gtk.BUTTONS_OK,
                               message_format=str)
    dialog.set_position(gtk.WIN_POS_CENTER)
    ret = dialog.run()
    dialog.destroy()
    return ret

def disp_info_dialog(self, str):
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
   			       type=gtk.MESSAGE_INFO,
			       buttons=gtk.BUTTONS_OK,
			       message_format=str)
    dialog.set_position(gtk.WIN_POS_CENTER)
    ret = dialog.run()
    dialog.destroy()
    return ret

def disp_verify_dialog(self, str):
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
			       type=gtk.MESSAGE_INFO,
			       buttons=gtk.BUTTONS_OK_CANCEL,
			       message_format=str)
    dialog.set_position(gtk.WIN_POS_CENTER)
    ret = dialog.run()
    dialog.destroy()
    if ret == gtk.RESPONSE_OK:
	return TRUE
    else:
	return FALSE


class DCSetupAbs:

    def on_delete_event(self, window, event=None):
        if disp_verify_dialog(self, _('Would you like to quit smbdcsetup?\n')):
            gtk.main_quit()

    def on_BtnNext_clicked(self, window):
	if not self.create_nextcomp():
	    disp_info_dialog(self, self.errstr)
	    return
        self.parent.go_next(self.nextcomp)

    def on_BtnPrev_clicked(self, window):
        self.parent.go_back()

    def show(self):
        self.win.show()

    def hide(self):
	self.win.hide()

    def create_nextcomp(self):
	self.nextcomp = None
	return FALSE

    def __init__(self, parent, widget):
        self.parent = parent
        self.xml = gtk.glade.XML(fname, widget, domain)
        self.win = self.xml.get_widget(widget)
        self.xml.signal_autoconnect(self)
	self.nextcomp = None
	self.errstr = _('Encountered unexpected error.\n')


class DCSetupMenu(DCSetupAbs):

    def on_dccreate_toggled(self, window):
	self.state = self.build

    def on_restore_toggled(self, window):
	self.state = self.restore

    def on_migrate_toggled(self, window):
	self.state = self.migrate

    def on_active_toggled(self, window):
	self.state = self.ads

    def restore_config_files(self):
	cmd = ''
	if os.path.isdir(backdir):
	    if os.path.isfile(backdir + 'smb.conf.backup'):
		cmd =	'/bin/cp ' + backdir + 'smb.conf.backup' + \
			' /etc/samba/smb.conf; '
	    if os.path.isfile(backdir + 'smbldap.conf.backup'):
		cmd =	cmd + '/bin/cp ' + backdir + 'smbldap.conf.backup' + \
			' /etc/samba/smbldap.conf; '
	    if os.path.isfile(backdir + 'smbldap_bind.conf.backup'):
		cmd =	cmd + '/bin/cp ' + backdir + 'smbldap_bind.conf.backup' + \
			' /etc/samba/smbldap_bind.conf; '
	    if os.path.isfile(backdir + 'slapd.conf.backup'):
		cmd =	cmd + '/bin/cp ' + backdir + 'slapd.conf.backup' + \
			' /etc/openldap/slapd.conf; '
	    if os.path.isfile(backdir + 'ldap.conf.backup'):
		cmd =	cmd + '/bin/cp ' + backdir + 'ldap.conf.backup' + \
			' /etc/openldap/ldap.conf; '
	    if os.path.isfile(backdir + 'ldap.conf2.backup'):
		cmd =	cmd + '/bin/cp ' + backdir + 'ldap.conf2.backup' + \
			' /etc/ldap.conf; '
	    if os.path.isfile(backdir + 'nsswitch.conf.backup'):
		cmd =	cmd + '/bin/cp ' + backdir + 'nsswitch.conf.backup' + \
			' /etc/nsswitch.conf; '
	    if os.path.isfile(backdir + 'system-auth.backup'):
		cmd =	cmd + '/bin/cp ' + backdir + 'system-auth.backup' + \
			' /etc/pam.d/system-auth'
	    if cmd != '':
		os.system(cmd)
		return TRUE
	return FALSE

    def create_nextcomp(self):
	if self.state == self.build:
	    self.nextcomp = DCSetupType(self.parent)
	    return TRUE
	elif self.state == self.migrate:
	    self.nextcomp = DCSetupDominfoMigrate(self.parent)
	    return TRUE
	elif self.state == self.ads:
            self.nextcomp = DCSetupWorkgroupADS(self.parent)
	    return TRUE
	else:
	    return FALSE

    def on_BtnNext_clicked(self, window):
	if self.state == self.restore:
	    if disp_verify_dialog(self, _('Would you like to restore config files?\n')):
		if self.restore_config_files():
		    disp_info_dialog(self, _('Config files successfully restored.\n'))
		    gtk.main_quit()
		else:
		    disp_info_dialog(self, _('Backup config files do not exist.\n'))
		    return
	    return
	DCSetupAbs.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_menu')
	self.build = 0
	self.restore = 1
	self.migrate = 2
	self.ads = 3
	self.state = self.build


class DCSetupType(DCSetupAbs):

    def on_pdcbut_toggled(self, window):
	self.state = self.pdc

    def on_bdcbut_toggled(self, window):
	self.state = self.bdc

    def on_stdbut_toggled(self, window):
	self.state = self.std

    def create_nextcomp(self):
	if self.state == self.pdc:
	    self.nextcomp = DCSetupDominfoPDC(self.parent)
	    return TRUE
	elif self.state == self.bdc:
	    self.nextcomp = DCSetupDominfoBDC(self.parent)
	    return TRUE
	elif self.state == self.std:
	    self.nextcomp = DCSetupWorkgroupStd(self.parent)
	    return TRUE
	else:
	    return FALSE

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_type')
	self.pdc = 0
	self.bdc = 1
	self.std = 2
	self.state = self.pdc


class DCSetupDominfo(DCSetupAbs):

    def on_BtnNext_clicked(self, window):
	domn = self.domain.get_text()
	biosn = self.netbios.get_text()
	if domn == '':
	    disp_err_dialog(self, _('Please specify a domain name.\n'))
	    return
	if biosn == '':
	    disp_err_dialog(self, _('Please specify a NetBIOS name.\n'))
	    return
	self.parent.set_domain_name(domn)
	self.parent.set_net_bios(biosn)
	DCSetupAbs.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_dominfo')
	self.domain = self.xml.get_widget('entryDomain')
	self.netbios = self.xml.get_widget('entryNetBIOS')
	(stat, f) = commands.getstatusoutput('/bin/hostname | cut -d. -f1')
	if stat == 0:
	    self.netbios.set_text(f)


class DCSetupDominfoPDC(DCSetupDominfo):

    def create_nextcomp(self):
	self.nextcomp = DCSetupLdapPDC(self.parent)
	return TRUE


class DCSetupDominfoBDC(DCSetupDominfo):

    def create_nextcomp(self):
	self.nextcomp = DCSetupLdapBDC(self.parent)
	return TRUE


class DCSetupDominfoMigrate(DCSetupDominfo):

    def create_nextcomp(self):
	self.nextcomp = DCSetupLdapMigrate(self.parent)
	return TRUE


class DCSetupWorkgroup(DCSetupAbs):

    def on_BtnNext_clicked(self, window):
	domn = self.workgrp.get_text()
	biosn = self.netbios.get_text()
	if domn == '':
	    disp_err_dialog(self, _('Please specify a workgroup name.\n'))
	    return
	if biosn == '':
	    disp_err_dialog(self, _('Please specify a NetBIOS name.\n'))
	    return
	self.parent.set_domain_name(domn)
	self.parent.set_net_bios(biosn)
	DCSetupAbs.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_workgroup')
	self.workgrp = self.xml.get_widget('entryWorkgroup')
	self.netbios = self.xml.get_widget('entryNetBIOS2')
	(stat, f) = commands.getstatusoutput('/bin/hostname | cut -d. -f1')
	if stat == 0:
	    self.netbios.set_text(f)


class DCSetupWorkgroupStd(DCSetupWorkgroup):

    def create_nextcomp(self):
	self.nextcomp = DCSetupStdBackend(self.parent)
	return TRUE


class DCSetupWorkgroupADS(DCSetupWorkgroup):

    def create_nextcomp(self):
	self.nextcomp = DCSetupADSParams(self.parent)
	return TRUE


class DCSetupStdBackend(DCSetupAbs):

    def on_ldapbut_toggled(self, window):
	self.ldapb = TRUE

    def on_tdbbut_toggled(self, window):
	self.ldapb = FALSE

    def create_nextcomp(self):
	if self.ldapb:
	    self.nextcomp = DCSetupLdapStd(self.parent)
	else:
	    self.nextcomp = DCSetupAdminStdTDB(self.parent)
	return TRUE

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_backend')
	self.ldapb = TRUE


class DCSetupADSParams(DCSetupAbs):

    def on_BtnNext_clicked(self, window):
	realm = self.realmb.get_text()
        kdcsrv = self.kdcsrvb.get_text()
	admin = self.adminb.get_text()
	adpass = self.adpassb.get_text()
        if realm == '':
            disp_err_dialog(self, _('Please specify ADS Realm.\n'))
            return
        if kdcsrv == '':
            disp_err_dialog(self, _('Please specify PDC Server.\n'))
            return
	if admin == '':
	    disp_err_dialog(self, _('Please specify ADS Administrator name.\n'))
	    return
	if adpass == '':
	    disp_err_dialog(self, _('Please specify ADS Administrator password.\n'))
	    return
        self.parent.set_ads_realm(realm)
        self.parent.set_ldap_master(kdcsrv)
	self.parent.set_ldap_admin(admin)
	self.parent.set_admin_passwd(adpass)
        DCSetupAbs.on_BtnNext_clicked(self, window)

    def create_nextcomp(self):
	self.nextcomp = DCSetupFilelistADS(self.parent)
	return TRUE

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_ads')
	self.realmb = self.xml.get_widget('entryRealm')
	self.kdcsrvb = self.xml.get_widget('entryKDC')
	self.adminb = self.xml.get_widget('entryADSAdmin')
	self.adpassb = self.xml.get_widget('entryADSPass')


class DCSetupLdap(DCSetupAbs):

    def is_valid_dns(self, str):
	return re.match(r'^[a-zA-Z0-9]([a-zA-Z0-9\-]*[a-zA-Z0-9])?(\.[a-zA-Z0-9]([a-zA-Z0-9\-]*[a-zA-Z0-9])?)+$', str)

    def on_BtnNext_clicked(self, window):
	self.ldapaddr = self.ldapip.get_text()
	self.domaindns = self.ldapsuffix.get_text()
	self.ldapsuff = ''
	p = ''
	if self.ldapaddr == '':
	    disp_err_dialog(self, _('Please specify LDAP server address.\n'))
	    return
	if self.domaindns == '':
	    disp_err_dialog(self, _('Please specify domain DNS.\n'))
	    return
	if not self.is_valid_dns(self.domaindns):
	    disp_err_dialog(self, _('Please specify valid domain DNS.\n'))
	    return
	part = string.split(self.domaindns, ".")
	cnt = 0
	for p in part:
	    if cnt != 0:
		self.ldapsuff = self.ldapsuff + ","
	    self.ldapsuff = self.ldapsuff + "dc=" + p
	    cnt += 1
	self.parent.set_ldap_address(self.ldapaddr)
	self.parent.set_ldap_suffix(self.ldapsuff)
        DCSetupAbs.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_ldap')
	self.ldapip = self.xml.get_widget('entryLDAPAddr')
	self.ldapsuffix = self.xml.get_widget('entryBaseSuffix')
	self.ldapaddr = ''
	self.domaindns = ''
	self.ldapsuff = ''


class DCSetupLdapPDC(DCSetupLdap):

    def create_nextcomp(self):
	if self.ldapaddr != 'localhost' and self.ldapaddr != '127.0.0.1':
	    self.errstr = _('Using remote LDAP server is not supported.\n')
	    return FALSE
	self.nextcomp = DCSetupAdminPDC(self.parent)
	return TRUE


class DCSetupLdapBDC(DCSetupLdap):

    def create_nextcomp(self):
	if self.ldapaddr != 'localhost' and self.ldapaddr != '127.0.0.1':
	    self.errstr = _('Using remote LDAP server is not supported.\n')
	    return FALSE
	self.nextcomp = DCSetupAdminBDC(self.parent)
	return TRUE


class DCSetupLdapStd(DCSetupLdap):

    def create_nextcomp(self):
	if self.ldapaddr != 'localhost' and self.ldapaddr != '127.0.0.1':
	    self.errstr = _('Using remote LDAP server is not supported.\n')
	    return FALSE
	self.nextcomp = DCSetupAdminStdLDAP(self.parent)
	return TRUE


class DCSetupLdapMigrate(DCSetupLdap):

    def create_nextcomp(self):
	if self.ldapaddr != 'localhost' and self.ldapaddr != '127.0.0.1':
	    self.errstr = _('Using remote LDAP server is not supported.\n')
	    return FALSE
	self.nextcomp = DCSetupMigrateServer(self.parent)
	return TRUE


class DCSetupMigrateServer(DCSetupAbs):

    def create_nextcomp(self):
	self.nextcomp = DCSetupAdminMigrate(self.parent)
	return TRUE

    def on_BtnNext_clicked(self, window):
	pdcname = self.currentpdc.get_text()
	if pdcname == '':
	    disp_err_dialog(self, _('Please specify PDC NetBIOS name.\n'))
	    return
	self.parent.set_current_pdc(pdcname)
	DCSetupAbs.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_migrate')
	self.currentpdc = self.xml.get_widget('entryCurrentPDC')


class DCSetupAdmin(DCSetupAbs):

    def get_admin_params(self):
	adminstr = self.admin.get_text()
	passstr = self.pass1.get_text()
	if adminstr == '':
	    disp_err_dialog(self, _('Please specify Administrator name.\n'))
	    return FALSE
	if passstr == '':
	    disp_err_dialog(self, _('Please specify Administrator password.\n'))
	    return FALSE
	if passstr != self.pass2.get_text():
	    disp_err_dialog(self, _('Confirmation password does not match with the password you specified.\n'))
	    return FALSE
	self.parent.set_ldap_admin(adminstr)
	self.parent.set_admin_passwd(passstr)
	return TRUE

    def on_BtnNext_clicked(self, window):
	if self.get_admin_params():
	    DCSetupAbs.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_admin')
	self.admin = self.xml.get_widget('entryAdmin')
	self.pass1 = self.xml.get_widget('entryPass')
	self.pass2 = self.xml.get_widget('entryPass2')


class DCSetupAdminPDC(DCSetupAdmin):

    def create_nextcomp(self):
	self.nextcomp = DCSetupRepPDC(self.parent)
	return TRUE


class DCSetupAdminBDC(DCSetupAdmin):

    def create_nextcomp(self):
	self.nextcomp = DCSetupRepBDC(self.parent)
	return TRUE


class DCSetupAdminStdLDAP(DCSetupAdmin):

    def create_nextcomp(self):
	self.nextcomp = DCSetupFilelistStdLDAP(self.parent)
	return TRUE


class DCSetupAdminStdTDB(DCSetupAdmin):

    def create_nextcomp(self):
	self.nextcomp = DCSetupLogStdTDB(self.parent)
	return TRUE

    def on_BtnNext_clicked(self, window):
	if self.get_admin_params():
	    if disp_verify_dialog(self, _('Would you like to update config files?\n')):
		DCSetupAbs.on_BtnNext_clicked(self, window)
		self.parent.do_job()


class DCSetupAdminMigrate(DCSetupAdmin):

    def create_nextcomp(self):
	self.nextcomp = DCSetupFilelistMigrate(self.parent)
	return TRUE


class DCSetupRepDialog:

    def run(self):
	list = []
	ret = self.win.run()
	while ret == gtk.RESPONSE_OK:
	    self.repservstr = self.repserv.get_text()
	    self.repuserstr = self.repuser.get_text()
	    self.reppassstr = self.reppass.get_text()
	    self.reppass2str = self.reppass2.get_text()
	    if self.repservstr == '':
		disp_err_dialog(self, _('Please specify slave server.\n'))
	    elif self.repuserstr == '':
		disp_err_dialog(self, _('Please specify replica user.\n'))
	    elif self.reppassstr == '':
		disp_err_dialog(self, _('Please specify password.\n'))
	    elif self.reppassstr != self.reppass2str:
		disp_err_dialog(self, _('Please specify same password.\n'))
	    else:
		list = [self.repservstr, self.repuserstr, self.reppassstr]
		break
	    ret = self.win.run()
	self.win.destroy()
	return list

    def __init__(self, parent, srv, usr, pas):
	self.parent = parent
	self.xml = gtk.glade.XML(fname, 'smbdcsetup_repdialog', domain)
	self.win = self.xml.get_widget('smbdcsetup_repdialog')
	self.repserv = self.xml.get_widget('entryRepServ')
	self.repuser = self.xml.get_widget('entryRepUser')
	self.reppass = self.xml.get_widget('entryRepPass')
	self.reppass2 = self.xml.get_widget('entryRepPass2')
	if srv != '':
	    self.repserv.set_text(srv)
	if usr != '':
	    self.repuser.set_text(usr)
	if pas != '':
	    self.reppass.set_text(pas)
	    self.reppass2.set_text(pas)
	self.xml.signal_autoconnect(self)


class DCSetupRepPDC(DCSetupAbs):

    def on_BtnAdd_clicked(self, window):
	repstr = 'cn=replica,' + self.parent.get_ldap_suffix()
        self.dlg = DCSetupRepDialog(self, '', repstr, '')
        retlist = self.dlg.run()
        if retlist != []:
            self.bdclist.append(retlist)
        return

    def on_BtnConfig_clicked(self, window):
        (model, iter) = self.listv.get_selection().get_selected()
        if iter == None:
            return
        serv = self.bdclist.get_value(iter, 0)
        user = self.bdclist.get_value(iter, 1)
        passwd = self.bdclist.get_value(iter, 2)
        self.dlg = DCSetupRepDialog(self, serv, user, passwd)
        retlist = self.dlg.run()
        if retlist != []:
            self.bdclist.insert_after(iter, retlist)
            self.bdclist.remove(iter)
        return

    def on_BtnDelete_clicked(self, window):
        (model, iter) = self.listv.get_selection().get_selected()
        if iter == None:
            return
        self.bdclist.remove(iter)
        return

    def create_nextcomp(self):
	self.nextcomp = DCSetupFilelistPDC(self.parent)
	return TRUE

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_reppdc')
	self.listv = self.xml.get_widget('bdclistView')
	self.addb = self.xml.get_widget('BdcBtnAdd')
	self.confb = self.xml.get_widget('BdcBtnConfig')
	self.delb = self.xml.get_widget('BdcBtnDel')
	self.listv.set_model(self.parent.dcBDCList)
	self.cell = gtk.CellRendererText()
	self.tvcolumn = gtk.TreeViewColumn('BDC name')
	self.listv.append_column(self.tvcolumn)
	self.cell = gtk.CellRendererText()
	self.tvcolumn.pack_start(self.cell, False)
	self.tvcolumn.add_attribute(self.cell, 'text', 0)
	self.bdclist = self.parent.get_bdc_list()


class DCSetupRepBDC(DCSetupAbs):

    def on_BtnNext_clicked(self, window):
	mst = self.masterbox.get_text()
	rp = self.repdnbox.get_text()
	p1 = self.repdbp1box.get_text()
	p2 = self.repdbp2box.get_text()
	if mst == '':
	    disp_err_dialog(self, _('Please specify Master LDAP address.\n'))
	    return
	if rp == '':
	    disp_err_dialog(self, _('Please specify LDAP replica DN.\n'))
	    return
	if p1 == '':
	    disp_err_dialog(self, _('Please specify LDAP replica password.\n'))
	    return
	if p1 != p2:
	    disp_err_dialog(self, _('Please specify the same password.\n'))
	    return
	self.parent.set_ldap_master(mst)
	self.parent.set_ldap_repdn(rp)
	self.parent.set_ldap_reppw(p1)
	DCSetupAbs.on_BtnNext_clicked(self, window)

    def create_nextcomp(self):
	self.nextcomp = DCSetupFilelistBDC(self.parent)
	return TRUE

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_repbdc')
	self.masterbox = self.xml.get_widget('entMaster')
	self.repdnbox = self.xml.get_widget('entRepDN')
	self.repdbp1box = self.xml.get_widget('entRepP1')
	self.repdbp2box = self.xml.get_widget('entRepP2')
	tmpstr = self.parent.get_ldap_suffix()
	if tmpstr != '':
	    tmpstr = 'cn=replica,' + tmpstr
	    self.repdnbox.set_text(tmpstr)


class DCSetupFilelist(DCSetupAbs):

    def on_BtnNext_clicked(self, window):
        if disp_verify_dialog(self, _('Would you like to update config files?\n')):
	    self.parent.set_file_list(self.modlist)
            DCSetupAbs.on_BtnNext_clicked(self, window)
	    self.parent.do_job()

    def on_autoBtn_toggled(self, window):
        self.file1.set_sensitive(FALSE)
	self.file2.set_sensitive(FALSE)
	self.file3.set_sensitive(FALSE)
	self.file4.set_sensitive(FALSE)
	self.file5.set_sensitive(FALSE)
	self.file6.set_sensitive(FALSE)
	self.file7.set_sensitive(FALSE)
        self.file1.set_active(TRUE)
        self.file2.set_active(TRUE)
        self.file3.set_active(TRUE)
        self.file4.set_active(TRUE)
        self.file5.set_active(TRUE)
        self.file6.set_active(TRUE)
        self.file7.set_active(TRUE)

    def on_manualBtn_toggled(self, window):
        self.file1.set_sensitive(TRUE)
        self.file2.set_sensitive(TRUE)
        self.file3.set_sensitive(TRUE)
        self.file4.set_sensitive(TRUE)
        self.file5.set_sensitive(TRUE)
        self.file6.set_sensitive(TRUE)
        self.file7.set_sensitive(TRUE)

    def on_fileCheck1_toggled(self, window):
        self.modlist[0] = self.file1.get_active()

    def on_fileCheck2_toggled(self, window):
        self.modlist[1] = self.file2.get_active()

    def on_fileCheck3_toggled(self, window):
        self.modlist[2] = self.file3.get_active()

    def on_fileCheck4_toggled(self, window):
        self.modlist[3] = self.file4.get_active()

    def on_fileCheck5_toggled(self, window):
        self.modlist[4] = self.file5.get_active()

    def on_fileCheck6_toggled(self, window):
        self.modlist[5] = self.file6.get_active()

    def on_fileCheck7_toggled(self, window):
        self.modlist[6] = self.file7.get_active()

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_filelist')
	self.auto = self.xml.get_widget('autoBtn')
	self.manual = self.xml.get_widget('manualBtn')
	self.file1 = self.xml.get_widget('fileCheck1')
	self.file2 = self.xml.get_widget('fileCheck2')
	self.file3 = self.xml.get_widget('fileCheck3')
	self.file4 = self.xml.get_widget('fileCheck4')
	self.file5 = self.xml.get_widget('fileCheck5')
	self.file6 = self.xml.get_widget('fileCheck6')
	self.file7 = self.xml.get_widget('fileCheck7')
	self.modlist = self.parent.get_file_list()


class DCSetupFilelistPDC(DCSetupFilelist):

    def create_nextcomp(self):
	self.nextcomp = DCSetupLogPDC(self.parent)
	return TRUE


class DCSetupFilelistBDC(DCSetupFilelist):

    def create_nextcomp(self):
	self.nextcomp = DCSetupLogBDC(self.parent)
	return TRUE


class DCSetupFilelistStdLDAP(DCSetupFilelist):

    def create_nextcomp(self):
	self.nextcomp = DCSetupLogStdLDAP(self.parent)
	return TRUE


class DCSetupFilelistMigrate(DCSetupFilelist):

    def create_nextcomp(self):
	self.nextcomp = DCSetupLogMigrate(self.parent)
	return TRUE


class DCSetupFilelistADS(DCSetupAbs):

    def on_BtnNext_clicked(self, window):
	if disp_verify_dialog(self, _('Would you like to update config files?\n')):
	    self.parent.set_file_list(self.modlist)
	    DCSetupAbs.on_BtnNext_clicked(self, window)
	    self.parent.do_job()

    def create_nextcomp(self):
	self.nextcomp = DCSetupLogADS(self.parent)
	return TRUE

    def on_autoBtn_toggled(self, window):
        self.file1.set_sensitive(FALSE)
        self.file2.set_sensitive(FALSE)
        self.file3.set_sensitive(FALSE)
        self.file4.set_sensitive(FALSE)
        self.file1.set_active(TRUE)
        self.file2.set_active(TRUE)
        self.file3.set_active(TRUE)
        self.file4.set_active(TRUE)

    def on_manualBtn_toggled(self, window):
	self.file1.set_sensitive(TRUE)
	self.file2.set_sensitive(TRUE)
	self.file3.set_sensitive(TRUE)
	self.file4.set_sensitive(TRUE)

    def on_fileCheck1_toggled(self, window):
	self.modlist[0] = self.file1.get_active()

    def on_fileCheck2_toggled(self, window):
	self.modlist[1] = self.file2.get_active()

    def on_fileCheck3_toggled(self, window):
	self.modlist[2] = self.file3.get_active()

    def on_fileCheck4_toggled(self, window):
	self.modlist[3] = self.file4.get_active()

    def __init__(self, parent):
	DCSetupAbs.__init__(self, parent, 'smbdcsetup_filelist_ads')
        self.auto = self.xml.get_widget('autoBtnADS')
        self.manual = self.xml.get_widget('manualBtnADS')
        self.file1 = self.xml.get_widget('fileCheck1ADS')
        self.file2 = self.xml.get_widget('fileCheck2ADS')
        self.file3 = self.xml.get_widget('fileCheck3ADS')
        self.file4 = self.xml.get_widget('fileCheck4ADS')
        self.modlist = [TRUE, TRUE, TRUE, TRUE]


class Backend:

    def stop_smb_daemon(self):
	if os.system('/sbin/pidof smbd > /dev/null 2>&1') == 0:
	    (stat, f) = commands.getstatusoutput('/sbin/service smb stop')
	    if stat == 0:
		self.buffer.insert(self.iter, _('Samba daemon(s) stopped.\n'))
	    else:
		disp_err_dialog(self, _('Failed to stop Samba server.\n'))
		return FALSE
	return TRUE

    def start_smb_daemon(self):
	(stat, f) = commands.getstatusoutput('/sbin/service smb start')
	if stat == 0:
	    self.buffer.insert(self.iter, _('Samba server started.\n'))
	    return TRUE
	else:
	    self.buffer.insert(self.iter, _('Failed to start Samba server.\n'))
	    return FALSE

    def start_smb_default(self):
	os.system('/sbin/chkconfig smb on')

    def stop_backend_daemon(self):
	return TRUE

    def start_backend_daemon(self):
	return TRUE

    def input_smb_passwd(self):
	cmd = '/bin/echo -e \"' + self.admin_passwd + '\n' + \
	      self.admin_passwd + '\n\" ' + \
	      ' | /usr/bin/smbpasswd -s ' + self.admin_name
	(stat, f) = commands.getstatusoutput(cmd)
	if stat == 0:
	    self.buffer.insert(self.iter, _('Updated ') + self.admin_name + _(' password.\n'))
	    return TRUE
	else:
	    disp_err_dialog(_('Failed to modify Administrator\'s password.\n'))
	    return FALSE

    def __init__(self, parent, buffer, iter):
	self.parent = parent
	self.buffer = buffer
	self.iter = iter
	self.admin_passwd = parent.get_admin_passwd()
	self.admin_name = parent.get_ldap_admin()


class BackendLdap(Backend):

    def backup_files(self):
	ret = disp_verify_dialog(self, \
		_('Would you like to take backup copies of config files?\n') + \
		_('Backup files will be saved at:\n') + '\n' + backdir + '\n')
	if ret:
	    if not os.path.isdir(backdir):
		os.system('/bin/mkdir -m700 -p ' + backdir)
	    cmd = '/bin/cp /etc/samba/smb.conf ' + backdir + 'smb.conf.backup; ' + \
		  '/bin/cp /etc/samba/smbldap.conf ' +  backdir + 'smbldap.conf.backup; ' + \
		  '/bin/cp /etc/samba/smbldap_bind.conf ' + backdir + 'smbldap_bind.conf.backup; ' + \
		  '/bin/cp /etc/openldap/slapd.conf ' + backdir + 'slapd.conf.backup; ' + \
		  '/bin/cp /etc/openldap/ldap.conf ' + backdir + 'ldap.conf.backup; ' + \
		  '/bin/cp /etc/ldap.conf ' + backdir + 'ldap.conf2.backup; ' + \
		  '/bin/cp /etc/nsswitch.conf ' + backdir + 'nsswitch.conf.backup; ' + \
		  '/bin/cp /etc/pam.d/system-auth ' + backdir + 'system-auth.backup'
	    os.system(cmd)
	    self.buffer.insert(self.iter, _('Backup files have been saved in \"') + tempdir + _('\"\n'))
	return

    def get_md5hash_passwd(self, str):
	(stat, f) = commands.getstatusoutput('/usr/sbin/slappasswd -h {MD5} -s ' + str)
	if stat == 0:
	    return f
	else:
	    return str

    def clear_backend_data(self):
	return FALSE

    def populate_backend_data(self):
	return FALSE

    def replace_smb_conf(self, templatef):
	file = '/etc/samba/smb.conf'
	cmd = '/bin/sed -e \'s/_DOMNAME_/' + self.domain_name + '/\' ' + \
		' -e \'s/_NBNAME_/' + self.netbios_name + '/\' ' + \
		' -e \'s/_LDAPSERV_/' + self.ldap_address + '/\' ' + \
		' -e \'s/_SUFFIX_/' + self.ldap_suffix + '/\' ' + \
		' -e \'s/_ADMIN_/' + self.admin_name + '/\' ' + \
		' -e \'s/_UNIXCHAR_/' + self.unix_char + '/\' ' + \
		' -e \'s/_DOSCHAR_/' + self.dos_char + '/\' ' + \
		templatef + ' > ' + file
	os.system(cmd)
	self.buffer.insert(self.iter, _('Generated /etc/samba/smb.conf\n'))
	(stat, f) = commands.getstatusoutput('/usr/bin/smbpasswd -w ' + \
			self.admin_passwd)
	self.buffer.insert(self.iter, f + '\n')

    def replace_smbldap_conf(self, template1):
	file1 = '/etc/samba/smbldap.conf'
	file2 = '/etc/samba/smbldap_bind.conf'
	template2 = tempdir + 'smbldap_bind.conf.template'
	cmd =	'/bin/sed -e \'s/_NBNAME_/' + self.netbios_name + '/\' ' + \
		' -e \'s/_DOMAIN_/' + self.domain_name + '/\' ' + \
		' -e \'s/_LDAPSERV_/' + self.ldap_address + '/\' ' + \
		' -e \'s/_SUFFIX_/' + self.ldap_suffix + '/\' ' + \
		' -e \'s/_MASTER_/' + self.master_server + '/\' ' + \
		' -e \'s/_SID_/' + self.sid + '/\' ' + template1 + ' ' + \
		'> ' + file1
	os.system(cmd)
	cmd =	'/bin/sed -e \'s/_LDAPSERV_/' + self.ldap_address + '/\' ' + \
               ' -e \'s/_SUFFIX_/' + self.ldap_suffix + '/\' ' + \
               ' -e \'s/_ADMIN_/' + self.admin_name + '/\' ' + \
               ' -e \'s/_PASSWD_/' + self.admin_passwd + '/\' ' + \
               template2 + ' > ' + file2
        os.system(cmd)
        self.buffer.insert(self.iter, _('Generated /etc/samba/smbldap.conf\n'))

    def replace_ldap_conf(self):
        file = '/etc/ldap.conf'
        template = tempdir + 'ldap.conf.template'
        cmd = '/bin/sed -e \'s/_LDAPSERV_/' + self.ldap_address + '/\' ' + \
              ' -e \'s/_SUFFIX_/' + self.ldap_suffix + '/\' ' + \
              template + ' > ' + file
        os.system(cmd)
        self.buffer.insert(self.iter, _('Generated /etc/ldap.conf\n'))

    def replace_ldap_conf2(self):
        file = '/etc/openldap/ldap.conf'
        template = tempdir + 'ldap.conf2.template'
        cmd = '/bin/sed -e \'s/_LDAPSERV_/' + self.ldap_address + '/\' ' + \
              ' -e \'s/_SUFFIX_/' + self.ldap_suffix + '/\' ' + \
              template + ' > ' + file
        os.system(cmd)
        self.buffer.insert(self.iter, _('Generated /etc/openldap/ldap.conf\n'))

    def add_extra_bit(self):
	return

    def replace_slapd_conf(self, templatef):
	file = '/etc/openldap/slapd.conf'
	passwd = self.get_md5hash_passwd(self.admin_passwd)
	cmd = '/bin/sed -e \'s/_SUFFIX_/' + self.ldap_suffix + '/\' ' + \
		  ' -e \'s/_ADMIN_/' + self.admin_name + '/\' ' + \
		  ' -e \'s/_REPLICA_/' + self.replica_dn + '/\' ' + \
		  ' -e \'s/_MASTER_/' + self.master_server + '/\' ' + \
		  templatef + ' > ' + file + '; /bin/echo -e \'rootpw\\t\\t' + passwd + \
		  '\' >> ' + file
	os.system(cmd)
	self.add_extra_bit()
	self.buffer.insert(self.iter, _('Generated /etc/openldap/slapd.conf\n'))

    def replace_system_auth(self):
	file = '/etc/pam.d/system-auth'
	template = tempdir + 'system-auth.ldap.template'
	os.system('/bin/cat ' + template + ' > ' + file)
	self.buffer.insert(self.iter, _('Generated /etc/pam.d/system-auth\n'))

    def replace_nsswitch(self):
	file = '/etc/nsswitch.conf'
	template = tempdir + 'nsswitch.conf.ldap.template'
	os.system('/bin/cat ' + template + ' > ' + file)
	self.buffer.insert(self.iter, _('Generated /etc/nsswitch.conf\n'))

    def acknowledge_sid(self, sid):
	self.sid = sid

    def is_updatable(self):
	return self.modlist[1]

    def __init__(self, parent, buffer, iter):
	Backend.__init__(self, parent, buffer, iter)
	self.unix_char = parent.get_unix_char()
	self.dos_char = parent.get_dos_char()
	self.domain_name = parent.get_domain_name()
	self.netbios_name = parent.get_net_bios()
	self.ldap_address = parent.get_ldap_address()
	self.ldap_suffix = parent.get_ldap_suffix()
	self.replica_dn = parent.get_ldap_repdn()
	self.replica_pw = parent.get_ldap_reppw()
	self.master_server = parent.get_ldap_master()
	self.modlist = parent.get_file_list()
	self.sid = ''


class BackendLdapLocal(BackendLdap):

    def clear_backend_data(self):
	os.system('/bin/rm -fr /var/lib/ldap/*')
	return TRUE

    def populate_backend_data(self):
	return TRUE

    def modify_files(self):
        if self.modlist[0]:
            self.replace_smb_conf()
        if self.modlist[1]:
            self.replace_smbldap_conf()
        if self.modlist[2]:
            self.replace_slapd_conf()
        if self.modlist[3]:
            self.replace_ldap_conf2()
        if self.modlist[4]:
            self.replace_ldap_conf()
        if self.modlist[5]:
            self.replace_nsswitch()
        if self.modlist[6]:
            self.replace_system_auth()
	return

    def stop_backend_daemon(self):
	if os.system('/sbin/pidof slapd > /dev/null 2>&1') == 0:
	    (stat, f) = commands.getstatusoutput('/sbin/service ldap stop')
	    if stat == 0:
		self.buffer.insert(self.iter, _('LDAP daemon(s) stopped.\n'))
	    else:
		disp_err_dialog(self, _('Failed to stop LDAP server.\n'))
		return FALSE
	return TRUE

    def start_backend_daemon(self):
	(stat, f) = commands.getstatusoutput('/sbin/service ldap start')
	if stat == 0:
	    self.buffer.insert(self.iter, _('LDAP server started.\n'))
	else:
	    disp_err_dialog(self, _('Failed to start LDAP server.\n'))
	    self.buffer.insert(self.iter, _('Failed to start LDAP server.\n'))
	    return FALSE
	return TRUE

    def start_backend_default(self):
	os.system('/sbin/chkconfig ldap on')


class BackendLdapLocalPDC(BackendLdapLocal):

    def add_extra_bit(self):
	iter = self.bdclist.get_iter_first()
        cmd = ''
        while iter != None:
            val1 = self.bdclist.get_value(iter, 0)
            val2 = self.bdclist.get_value(iter, 1)
            val3 = self.bdclist.get_value(iter, 2)
            cmd = cmd + 'replica host=' + val1 + '\\n\\t' + \
                  'binddn=\"' + val2 + '\"\\n\\t' + \
                  'bindmethod=simple\\n\\t' + \
                  'credentials=' + val3 + '\\n\\n'
            iter = self.bdclist.iter_next(iter)
        if cmd != '':
            cmd = '/bin/echo -e \"\\n' + cmd + \
                  'replogfile\\t/var/lib/ldap/master-replica.log\" ' + \
                  ' >> /etc/openldap/slapd.conf'
        os.system(cmd)

    def replace_smb_conf(self):
	BackendLdapLocal.replace_smb_conf(self, tempdir + 'smb.conf.master.template')

    def replace_smbldap_conf(self):
	BackendLdapLocal.replace_smbldap_conf(self, tempdir + 'smbldap.conf.master.template')

    def replace_slapd_conf(self):
	BackendLdapLocal.replace_slapd_conf(self, tempdir + 'slapd.conf.master.template')

    def input_smb_passwd(self):
	if Backend.input_smb_passwd(self):
	    os.system('perl -pi -e \'s|^(rootpw\s*).*||\' /etc/openldap/slapd.conf')
	    return TRUE
	else:
	    return FALSE

    def populate_backend_data(self):
	(stat, f) = commands.getstatusoutput('/usr/sbin/smbldap-populate' + \
		    ' -a ' + self.admin_name + ' -b Guest')
	self.buffer.insert(self.iter, f + '\n')
	if stat != 0:
	    self.buffer.insert(self.iter, _('Failed to populate data.\n'))
	    return FALSE
	(stat, f) = commands.getstatusoutput('/usr/sbin/smbldap-usermod' + \
		    ' -F \'\' ' + self.admin_name)
	if stat != 0:
	    self.buffer.insert(self.iter, _('Failed to modify user ') + self.admin_name + '\n')
	    return FALSE
	return TRUE

    def __init__(self, parent, buffer, iter):
	BackendLdapLocal.__init__(self, parent, buffer, iter)
	self.bdclist = parent.get_bdc_list()


class BackendLdapLocalBDC(BackendLdapLocal):

    def replace_smb_conf(self):
        BackendLdapLocal.replace_smb_conf(self, tempdir + 'smb.conf.slave.template')

    def replace_smbldap_conf(self):
        BackendLdapLocal.replace_smbldap_conf(self, tempdir + 'smbldap.conf.slave.template')

    def replace_slapd_conf(self):
        BackendLdapLocal.replace_slapd_conf(self, tempdir + 'slapd.conf.slave.template')

    def populate_backend_data(self):
	cmd = '/usr/bin/ldapsearch -x -LLL -h ' + self.master_server + \
	      ' -D \'uid=' + self.admin_name + ',ou=Users,' + self.ldap_suffix + \
	      '\' -w ' + self.admin_passwd + ' | /usr/sbin/slapadd'
	(stat, f) = commands.getstatusoutput(cmd)
	if stat != 0:
	    self.buffer.insert(self.iter, f)
	    self.buffer.insert(self.iter, _('Failed to populate data.\n'))
	    return FALSE
	os.system('/bin/chown -R ldap:ldap /var/lib/ldap')
	return TRUE


class BackendLdapLocalStd(BackendLdapLocal):

    def replace_smb_conf(self):
	BackendLdapLocal.replace_smb_conf(self, tempdir + 'smb.conf.standalone.template')

    def replace_smbldap_conf(self):
	BackendLdapLocal.replace_smbldap_conf(self, tempdir + 'smbldap.conf.master.template')

    def replace_slapd_conf(self):
	BackendLdapLocal.replace_slapd_conf(self, tempdir + 'slapd.conf.master.template')

    def input_smb_passwd(self):
	if Backend.input_smb_passwd(self):
	    os.system('perl -pi -e \'s|^(rootpw\s*).*||\' /etc/openldap/slapd.conf')
	    return TRUE
	else:
	    return FALSE

    def populate_backend_data(self):
	(stat, f) = commands.getstatusoutput('/usr/sbin/smbldap-populate' + \
			' -a ' + self.admin_name + ' -b Guest')
	self.buffer.insert(self.iter, f + '\n')
	if stat != 0:
	    self.buffer.insert(self.iter, _('Failed to populate data.\n'))
	    return FALSE
	(stat, f) = commands.getstatusoutput('/usr/sbin/smbldap-usermod' + \
			' -F \'\' ' + self.admin_name)
	if stat != 0:
	    self.buffer.insert(self.iter, _('Failed to modify user ') + self.admin_name + '\n')
	    return FALSE
	return TRUE


class BackendLdapLocalMigrate(BackendLdapLocal):

    def replace_smb_conf(self):
        BackendLdapLocal.replace_smb_conf(self, tempdir + 'smb.conf.slave.template')

    def replace_smb_conf2(self):
	BackendLdapLocal.replace_smb_conf(self, tempdir + 'smb.conf.master.template')

    def replace_smbldap_conf(self):
        BackendLdapLocal.replace_smbldap_conf(self, tempdir + 'smbldap.conf.master.template')

    def replace_slapd_conf(self):
	BackendLdapLocal.replace_slapd_conf(self, tempdir + 'slapd.conf.master.template')

    def populate_backend_data(self):
	part = string.split(self.ldap_suffix, ",", 1)
	tstr = string.split(part[0], "=", 1)
	template = tempdir + 'initialize.template'
	cmd = '/bin/sed -e \'s/_DOMNAME_/' + self.domain_name + '/\' ' + \
	      ' -e \'s/_BASE_/' + self.ldap_suffix + '/\' ' + \
	      ' -e \'s/_PART_/' + tstr[1] + '/\' ' + \
	      ' -e \'s/_SID_/' + self.sid + '/\' ' + template + ' ' + \
	      ' | /usr/sbin/slapadd '
	(stat, f) = commands.getstatusoutput(cmd)
	if stat != 0:
	    disp_err_dialog(self, _('Failed to initialise LDAP data\n'))
	    self.buffer.insert(self.iter, f)
	    return FALSE
	os.system('/bin/chown -R ldap:ldap /var/lib/ldap')
	if not self.start_backend_daemon():
	    return FALSE
	(stat, f) = commands.getstatusoutput('/usr/bin/net rpc vampire ' + \
			' -S ' + self.parent.dcCurrentPDC + \
			' -U ' + self.parent.dcLdapAdmin + '%' + self.parent.dcPassword)
	self.buffer.insert(self.iter, f)
	if stat != 0:
	    disp_err_dialog(self, _('Failed to extract data from the NT domain.\n'))
	    return FALSE
	else:
            return TRUE


class BackendLdapRemote(BackendLdap):

    def clear_backend_data(self):
	cmd =	'/usr/bin/ldapsearch -x -h ' + self.ldap_address + \
		' | /bin/grep \'^dn: \' | /bin/sed \'s/^dn: //\' | /usr/bin/tac | ' + \
		'/usr/bin/ldapdelete -x -h ' + self.ldap_address + \
		' -D \'uid=' + self.admin_name + \
		',ou=Users,' + self.ldap_suffix + '\' -w ' + \
		self.admin_passwd
	(stat, f) = commands.getstatusoutput(cmd)
	self.buffer.insert(self.iter, f)
	if stat != 0:
	    disp_err_dialog(self, _('Failed to delete LDAP data.\n'))
	    self.buffer.insert(self.iter, _('Failed to delete LDAP data.\n'))
	    return FALSE
	return TRUE

    def populdate_backend_data(self):
	cmd =	'/usr/bin/ldapsearch -x -LLL -h ' + \
		self.master_server + ' -D \'uid=' + self.admin_name + \
		',ou=Users,' + self.ldap_suffix + '\' -w ' + self.admin_passwd + \
		' | /usr/sbin/slapadd'
	(stat, f) = commands.getstatusoutput(cmd)
	if stat != 0:
	    self.buffer.insert(self.iter, f)
	    return FALSE
	os.system('/bin/chown -R ldap:ldap /var/lib/ldap')
	return TRUE


class BackendTDB(Backend):

    def backup_files(self):
	ret = disp_verify_dialog(self, \
		_('Would you like to take backup copies of config files?\n') + \
		_('Backup files will be saved at:\n') + '\n' + backdir + '\n')
	if ret:
	    if not os.path.isdir(backdir):
		os.system('/bin/mkdir -m700 -p ' + backdir)
	    cmd = '/bin/cp /etc/samba/smb.conf ' + backdir + 'smb.conf.backup; '
	    os.system(cmd)
	    self.buffer.insert(self.iter, _('Backup files have been saved in \"') + tempdir + _('\"\n'))
	return

    def clear_backend_data(self):
	os.system('/bin/rm -f /etc/samba/passwd.tdb')

    def populate_backend_data(self):
	try:
	    dummy = pwd.getpwnam('administrator')
	except KeyError:
	    saltlist = string.ascii_letters + string.digits + '/.'
	    crypted = crypt.crypt(self.admin_passwd, random.choice(saltlist) + random.choice(saltlist))
	    (stat, f) = commands.getstatusoutput('/usr/sbin/useradd -p ' + crypted + ' administrator')
	    self.buffer.insert(self.iter, f + '\n')
	    if stat != 0:
		err = _('Failed to add user Administrator.\n')
		self.buffer.insert(self.iter, err)
		return FALSE
	cmd = '/bin/echo -e \"' + self.admin_passwd + '\n' + \
		self.admin_passwd + '\n\" ' + \
		' | /usr/bin/smbpasswd -a -s ' + self.admin_name
	(stat, f) = commands.getstatusoutput(cmd)
	self.buffer.insert(self.iter, f + '\n')
	if stat != 0:
	    err = _('Failed to add user Administrator.\n')
	    self.buffer.insert(self.iter, err)
	    return FALSE
	return TRUE

    def modify_files(self):
	srcf = tempdir + 'smb.conf.tdbsam.template'
	dest = '/etc/samba/smb.conf'
	cmd = '/bin/sed -e \'s/_DOMNAME_/' + self.domain_name + '/\' ' + \
		' -e \'s/_NBNAME_/' + self.netbios_name + '/\' ' + \
		' -e \'s/_ADMIN_/' + self.admin_name + '/\' ' + \
		' -e \'s/_UNIXCHAR_/' + self.unix_char + '/\' ' + \
		' -e \'s/_DOSCHAR_/' + self.dos_char + '/\' ' + \
		srcf + ' > ' + dest
	os.system(cmd)
	self.buffer.insert(self.iter, _('Generated /etc/samba/smb.conf\n'))

    def is_updatable(self):
	return TRUE

    def __init__(self, parent, buffer, iter):
	Backend.__init__(self, parent, buffer, iter)
	self.unix_char = parent.get_unix_char()
	self.dos_char = parent.get_dos_char()
	self.domain_name = parent.get_domain_name()
	self.netbios_name = parent.get_net_bios()


class BackendADS(Backend):

    def backup_files(self):
	ret = disp_verify_dialog(self, \
	      _('Would you like to take backup copies of config files?\n') + \
	      _('Backup files will be saved at:\n') + '\n' + backdir + '\n')
	if ret:
	    if not os.path.isdir(backdir):
		os.system('/bin/mkdir -m700 -p ' + backdir)
	    cmd = '/bin/cp /etc/samba/smb.conf ' + backdir + 'smb.conf.backup; ' + \
		      '/bin/cp /etc/krb5.conf ' + backdir + 'krb5.conf.backup; ' + \
		      '/bin/cp /etc/pam.d/system-auth ' + backdir + 'system-auth.backup; ' + \
		      '/bin/cp /etc/nsswitch.conf ' + backdir + 'nsswitch.conf.backup'
	    os.system(cmd)
	    self.buffer.insert(self.iter, _('Backup files have been saved in \"') + tempdir + _('\"\n'))
        return

    def clear_backend_data(self):
	os.system('/bin/rm -fr /var/cache/samba/winbindd_*')

    def replace_smb_conf(self):
	file = '/etc/samba/smb.conf'
	template = tempdir + 'smb.conf.ads.template'
	cmd = '/bin/sed -e \'s/_DOMNAME_/' + self.domain_name + '/\' ' + \
		' -e \'s/_NBNAME_/' + self.netbios_name + '/\' ' + \
		' -e \'s/_ADMIN_/' + self.admin_name + '/\' ' + \
		' -e \'s/_KDCSERV_/' + self.master_server + '/\' ' + \
		' -e \'s/_REALM_/' + self.ads_realm + '/\' ' + \
		' -e \'s/_UNIXCHAR_/' + self.unix_char + '/\' ' + \
		' -e \'s/_DOSCHAR_/' + self.dos_char + '/\' ' + \
                template + ' > ' + file
	os.system(cmd)
	self.buffer.insert(self.iter, _('Generated /etc/samba/smb.conf\n'))

    def replace_krb5_conf(self):
	file = '/etc/krb5.conf'
	template = tempdir + 'krb5.conf.template'
	dns = string.lower(self.ads_realm)
	cmd = '/bin/sed -e \'s/_REALM_/' + self.ads_realm + '/\' ' + \
		' -e \'s/_KDCSERV_/' + self.master_server + '/\' ' + \
		' -e \'s/_DNS_/' + dns + '/\' ' + \
		template + ' > ' + file
	os.system(cmd)
	self.buffer.insert(self.iter, _('Generated /etc/krb5.conf\n'))

    def replace_system_auth(self):
	file = '/etc/pam.d/system-auth'
	template = tempdir + 'system-auth.winbind.template'
        os.system('/bin/cat ' + template + ' > ' + file)
        self.buffer.insert(self.iter, _('Generated /etc/pam.d/system-auth\n'))

    def replace_nsswitch(self):
	file = '/etc/nsswitch.conf'
	template = tempdir + 'nsswitch.conf.winbind.template'
	os.system('/bin/cat ' + template + ' > ' + file)
	self.buffer.insert(self.iter, _('Generated /etc/nsswitch.conf\n'))

    def modify_files(self):
	if self.modlist[0]:
	    self.replace_smb_conf()
	if self.modlist[1]:
	    self.replace_krb5_conf()
	if self.modlist[2]:
	    self.replace_nsswitch()
	if self.modlist[3]:
	    self.replace_system_auth()

    def stop_backend_daemon(self):
	if os.system('/sbin/pidof winbindd > /dev/null 2>&1') == 0:
	    (stat, f) = commands.getstatusoutput('/sbin/service winbind stop')
	    if stat == 0:
		self.buffer.insert(self.iter, _('Winbind daemon stopped.\n'))
	    else:
		disp_err_dialog(self, _('Failed to stop Winbind server.\n'))
		return FALSE
	return TRUE

    def start_backend_daemon(self):
	(stat, f) = commands.getstatusoutput('/sbin/service winbind start')
	if stat == 0:
	    self.buffer.insert(self.iter, _('Winbind server started.\n'))
	    return TRUE
	else:
	    self.buffer.insert(self.iter, _('Failed to start Winbind server.\n'))
	    return FALSE

    def start_backend_default(self):
	os.system('/sbin/chkconfig winbind on')

    def populate_backend_data(self):
	cmd = '/usr/bin/expect -c \"spawn /usr/kerberos/bin/kinit ' + self.admin_name + \
	      '; expect ' + self.admin_name + '@' + self.ads_realm + ': ; ' + \
	      'send ' + self.admin_passwd + '\n ; expect eof\"'
	(stat, f) = commands.getstatusoutput(cmd)
	self.buffer.insert(self.iter, f)
	if stat != 0:
	    self.buffer.insert(self.iter, _('Failed to retrieve kerberos ticket.\n'))
	    return FALSE
	cmd = '/usr/bin/net ads join -U ' + self.admin_name + '%' + self.admin_passwd
	(stat, f) = commands.getstatusoutput(cmd)
	self.buffer.insert(self.iter, f)
	if stat != 0:
	    self.buffer.insert(self.iter, _('Failed to join ADS.\n'))
	    return FALSE
	return TRUE

    def is_updatable(self):
	return TRUE

    def __init__(self, parent, buffer, iter):
	Backend.__init__(self, parent, buffer, iter)
	self.unix_char = parent.get_unix_char()
	self.dos_char = parent.get_dos_char()
	self.domain_name = parent.get_domain_name()
	self.netbios_name = parent.get_net_bios()
	self.master_server = parent.get_ldap_master()
	self.ads_realm = parent.get_ads_realm() 
	self.modlist = parent.get_file_list()


class DCSetupLog(DCSetupAbs):

    def populate_data(self):
	return (TRUE, '')

    def do_job(self):
        self.backend.backup_files()
	if not self.backend.stop_smb_daemon():
	    return
	if not self.backend.stop_backend_daemon():
	    return
        self.backend.modify_files()
	if self.backend.is_updatable() and disp_verify_dialog(self, _('Would you like to populate data?\n')):
	    (ret, out) = self.populate_data()
	    if not ret:
                self.progress.set_text(out)
                return
            self.progress.set_text(out)
	else:
	    self.progress.set_text(_('Please generate data by yourself.\n'))
        self.canc.set_sensitive(FALSE)
        self.prev.set_sensitive(FALSE)
        self.term.set_sensitive(TRUE)

    def get_master_sid(self):
	f = ''
	attempt = 1
	while attempt <= 30:
	    self.buffer.insert(self.iter, str(attempt) + 'th try\n')
	    (stat, f) = commands.getstatusoutput('/bin/sleep 1; ' + \
			'/usr/bin/net getlocalsid 2> /dev/null | ' + \
			'/bin/grep \'SID for domain\' | ' + \
			'/bin/cut -d: -f2 | /bin/sed \'s/ //\'')
	    if stat == 0:
		break
	    else:
		attempt = attempt + 1
	if stat != 0:
	    disp_err_dialog(_('Failed to retrieve SID\n'))
	    return FALSE
	else:
	    self.sid = f
	    return TRUE

    def get_slave_sid(self):
	f = ''
	attempt = 1
	self.buffer.insert(self.iter, _('Retrieving domain SID...\n'))
	while attempt <= 30:
	    self.buffer.insert(self.iter, str(attempt) + 'th try\n')
	    (stat, f) = commands.getstatusoutput('/bin/sleep 1; ' + \
			'/usr/bin/net rpc getsid 2> /dev/null | ' + \
			'/bin/grep \'^Storing SID\' | ' + \
			'/bin/cut -d\' \' -f3')
	    if stat == 0:
		break
	    else:
		attempt = attempt + 1
	if stat != 0:
	    disp_err_dialog(_('Failed to retrieve SID\n'))
	    return FALSE
	else:
	    self.sid = f
	    self.buffer.insert(self.iter, _('SID obtained: ') + self.sid + '\n')
	    self.backend.acknowledge_sid(self.sid)
	    return TRUE

    def on_BtnNext_clicked(self, window):
	return

    def __init__(self, parent):
        DCSetupAbs.__init__(self, parent, 'smbdcsetup_log')
        self.canc = self.xml.get_widget('cancBut')
        self.prev = self.xml.get_widget('prevBut')
        self.term = self.xml.get_widget('termBut')
        self.progress = self.xml.get_widget('progress_label')
        self.logview = self.xml.get_widget('logview')
        self.buffer = self.logview.get_buffer()
        self.iter = self.buffer.get_start_iter()
        self.sid = 'S-1-5-21-3450389124-3982801727-2332141301'
	self.admin_name = parent.get_ldap_admin()
	self.admin_passwd = parent.get_admin_passwd()
	self.backend = None


class DCSetupLogPDC(DCSetupLog):

    def populate_data(self):
	succstr = _('Successfully created NT domain.\n')
	failstr = _('Failed to populate data. Please refer to the log below for more information.\n')
	self.backend.clear_backend_data()
	self.backend.start_backend_daemon()
	self.get_master_sid()
	self.backend.acknowledge_sid(self.sid)
	self.backend.replace_smbldap_conf()
	os.system('/usr/bin/net setlocalsid ' + self.sid)
	if not self.backend.populate_backend_data():
	    return (FALSE, failstr)
	if not self.backend.input_smb_passwd():
	    return (FALSE, failstr)
	if self.backend.start_smb_daemon():
	    self.backend.start_smb_default()
	    self.backend.start_backend_default()
	    return (TRUE, succstr)
	else:
	    return (FALSE, failstr)

    def __init__(self, parent):
	DCSetupLog.__init__(self, parent)
	self.backend = BackendLdapLocalPDC(parent, self.buffer, self.iter)


class DCSetupLogBDC(DCSetupLog):

    def populate_data(self):
	succstr = _('Successfully created NT domain.\n')
	failstr = _('Failed to populate data. Please refer to the log below for more information.\n')
	self.backend.clear_backend_data()
	if not self.backend.populate_backend_data():
	    return (FALSE, failstr)
	if not self.get_slave_sid():
	    return (FALSE, failstr)
	os.system('/usr/bin/net setlocalsid ' + self.sid)
	self.backend.replace_smbldap_conf()
	self.backend.start_backend_daemon()
	if self.backend.start_smb_daemon():
	    self.backend.start_smb_default()
	    self.backend.start_backend_default()
	    return (TRUE, succstr)
        else:
	    return (FALSE, failstr)

    def __init__(self, parent):
	DCSetupLog.__init__(self, parent)
	self.backend = BackendLdapLocalBDC(parent, self.buffer, self.iter)


class DCSetupLogStdLDAP(DCSetupLog):

    def populate_data(self):
	succstr = _('Successfully created standalone file server.\n')
	failstr = _('Failed to populate data. Please refer to the log below for more information.\n')
	self.backend.clear_backend_data()
	self.backend.start_backend_daemon()
	if not self.backend.populate_backend_data():
	    return (FALSE, failstr)
	if not self.backend.input_smb_passwd():
	    return (FALSE, failstr)
	if self.backend.start_smb_daemon():
	    self.backend.start_smb_default()
	    self.backend.start_backend_default()
	    return (TRUE, succstr)
	else:
	    return (FALSE, failstr)

    def __init__(self, parent):
	DCSetupLog.__init__(self, parent)
	self.backend = BackendLdapLocalStd(parent, self.buffer, self.iter)


class DCSetupLogStdTDB(DCSetupLog):

    def populate_data(self):
	succstr = _('Successfully created standalone file server.\n')
	failstr = _('Failed to populate data. Please refer to the log below for more information.\n')
	self.backend.clear_backend_data()
	stat = self.backend.populate_backend_data()
	if not stat:
	    return (FALSE, failstr)
	if self.backend.start_smb_daemon():
	    self.backend.start_smb_default()
	    return (TRUE, succstr)
	else:
	    return (FALSE, failstr)

    def __init__(self, parent):
	DCSetupLog.__init__(self, parent)
	self.backend = BackendTDB(parent, self.buffer, self.iter)


class DCSetupLogMigrate(DCSetupLog):

    def populate_data(self):
	self.progress.set_text(_('Migrating data from a NT domain.\n'))
	succstr = _('Successfully migrated NT domain data.\n')
	failstr = _('Failed to populate data. Please refer to the log below for more information.\n')
	self.backend.clear_backend_data()
	if not self.get_slave_sid():
	    return (FALSE, failstr)
	stat = self.backend.populate_backend_data()
        if not stat:
	    return (FALSE, failstr)
	self.backend.replace_smb_conf2()
	ret = disp_verify_dialog(self, _('You need to shutdown the current PDC server in order to start using Samba server as a new PDC. Would you like to start Samba server now?\n'))
	if ret:
	    if self.backend.start_smb_daemon():
        	self.backend.start_smb_default()
		self.backend.start_backend_default()
		return (TRUE, succstr)
	    else:
		return (FALSE, failstr)
	else:
	    outstr = _('Please shutdown NT server first before you start Samba service.\n')
	    return (TRUE, succstr)

    def __init__(self, parent):
	DCSetupLog.__init__(self, parent)
	self.backend = BackendLdapLocalMigrate(parent, self.buffer, self.iter)


class DCSetupLogADS(DCSetupLog):

    def populate_data(self):
	self.progress.set_text(_('Joining in to ADS.\n'))
        succstr = _('Successfully joined to ADS.\n')
	failstr = _('Failed to join ADS.\n')
        self.backend.clear_backend_data()
	stat = self.backend.populate_backend_data()
	if not stat:
	    return (FALSE, failstr)
	if self.backend.start_smb_daemon() and self.backend.start_backend_daemon():
	    self.backend.start_smb_default()
	    self.backend.start_backend_default()
	    return (TRUE, succstr)
	else:
	    return (FALSE, failstr)

    def __init__(self, parent):
	DCSetupLog.__init__(self, parent)
	self.backend = BackendADS(parent, self.buffer, self.iter)


class DCSetupMain:

    def go_back(self):
	self.current.hide()
        self.current = self.backlist.pop()
        self.current.show()

    def go_next(self, next):
	self.current.hide()
        self.backlist.append(self.current)
        self.current = next
        self.current.show()

    def do_job(self):
	self.current.do_job()

    def get_unix_char(self):
	return self.dcUnixChar

    def get_dos_char(self):
	return self.dcDosChar

    def get_bdc_list(self):
	return self.dcBDCList

    def get_ldap_address(self):
	return self.dcLdapServer

    def set_ldap_address(self, str):
	self.dcLdapServer = str

    def get_ldap_suffix(self):
	return self.dcLdapSuffix

    def set_ldap_suffix(self, str):
	self.dcLdapSuffix = str

    def get_ldap_admin(self):
	return self.dcLdapAdmin

    def set_ldap_admin(self, str):
	self.dcLdapAdmin = str

    def get_net_bios(self):
	return self.dcNetBIOS

    def set_net_bios(self, str):
	self.dcNetBIOS = str

    def get_domain_name(self):
	return self.dcDomainName

    def set_domain_name(self, str):
	self.dcDomainName = str

    def get_admin_passwd(self):
	return self.dcPassword

    def set_admin_passwd(self, str):
	self.dcPassword = str

    def get_ldap_master(self):
	return self.dcLdapMaster

    def set_ldap_master(self, str):
	self.dcLdapMaster = str

    def get_ldap_repdn(self):
	return self.dcLdapRepDN

    def set_ldap_repdn(self, str):
	self.dcLdapRepDN = str

    def get_ldap_reppw(self):
	return self.dcLdapRepPW

    def set_ldap_reppw(self, str):
	self.dcLdapRepPW = str

    def get_current_pdc(self):
	return self.dcCurrentPDC

    def set_current_pdc(self, str):
	self.dcCurrentPDC = str

    def get_ads_realm(self):
	return self.dcADSRealm

    def set_ads_realm(self, str):
	self.dcADSRealm = str

    def get_file_list(self):
	return self.modlist

    def set_file_list(self, list):
	self.modlist = list

    def set_locale(self):
	(lang, encoding) = locale.getlocale()
	if lang == 'ja_JP':
	    self.dcDosChar = 'CP932'
	    if encoding == 'eucJP':
		self.dcUnixChar = 'EUCJP-MS'
	    elif encoding == 'SJIS':
		self.dcUnixChar = 'CP932'
	    else:
		self.dcUnixChar = 'UTF-8'
	elif lang == 'ko_KR':
	    self.dcDosChar = 'CP949'
	    if encoding == 'euckr':
		self.dcUnixChar = 'EUC-KR'
	    else:
		self.dcUnixChar = 'UTF-8'
	elif lang == 'zh_CN':
	    self.dcUnixChar = 'GB18030'
	    self.dcDosChar = 'CP936'
	elif lang == 'zh_TW':
	    self.dcUnixChar = 'BIG5'
	    self.dcDosChar = 'CP950'
	else:
	    self.dcUnixChar = 'UTF-8'
	    self.dcDosChar = 'UTF-8'

    def __init__(self):
	self.backlist = []
	self.dcUnixChar = 'UTF-8'
	self.dcDosChar = 'UTF-8'
	self.dcLdapServer = ''
	self.dcLdapSuffix = ''
	self.dcLdapAdmin = ''
	self.dcNetBIOS = ''
	self.dcDomainName = ''
	self.dcPassword = ''
	self.dcLdapMaster = ''
	self.dcLdapRepDN = ''
	self.dcLdapRepPW = ''
	self.dcCurrentPDC = ''
	self.dcADSRealm = ''
	self.backend = None
	self.modlist = [TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE]
	self.dcBDCList = gtk.ListStore(str, str, str)
	self.set_locale()
	self.current = DCSetupMenu(self)
	self.current.show()


if os.getuid() != 0 and os.geteuid() != 0:
    sys.exit('smbdcsetup can only be invoked by root user')

smbdcsetup = DCSetupMain()
signal.signal(signal.SIGINT, gtk.main_quit)
gtk.main()
